local name, mNameplates = ...

local DEBUG = mNameplates.DEBUG
local Debug = mNameplates.Debug
local expect = Debug.expect

Debug:AddGlobal("addon", mNameplates)

mNameplates.DisplayIcon = [[Interface\Icons\inv_shield_09]]

local LSM = LibStub("LibSharedMedia-3.0")
local LNP = LibStub("LibNameplate-1.0")

LSM:Register("font", "accid", [[Interface\AddOns\mNameplates\Media\accid.ttf]])
LSM:Register("statusbar", "HalD", [[Interface\AddOns\mNameplates\Media\HalD.tga]])
LSM:Register("border", "Glowing", [[Interface\AddOns\mNameplates\Media\Glowing.tga]])

local db
local InCombat = false
local clickAreaVisible = false
local PlateProt = {}
local PLateProtMT = {__index = PlateProt}
local Plates = {}
local InactivePlates = {}

Debug:AddGlobal("Plates", Plates)
Debug:AddGlobal("InactivePlates", InactivePlates)

function mNameplates:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			Width = 125,
			Height = 25,
			
			BackgroundColor = {0, 0, 0, 0.5},
			BorderColor = {1, 1, 1, 1},
			BorderColorThreat = {1, 0 ,0, 1},
			Border = "Beuty Case",
		}
	}

	self.db  = LibStub("AceDB-3.0"):New("mNameplates2DB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self.Callbacks = LibStub("CallbackHandler-1.0"):New(self)	
	
	self:RegisterChatCommand("mn", "ToggleConfig")
	
	LNP.RegisterCallback(self, "LibNameplate_NewNameplate")
	LNP.RegisterCallback(self, "LibNameplate_RecycleNameplate")
	LNP.RegisterCallback(self, "LibNameplate_MouseoverNameplate")
	LNP.RegisterCallback(self, "LibNameplate_FoundGUID")
	LNP.RegisterCallback(self, "LibNameplate_HealthChange")
	LNP.RegisterCallback(self, "LibNameplate_ThreatChange")
	
	for plate in LNP:IteratePlates() do
		self:LibNameplate_NewNameplate("LibNameplate_NewNameplate", plate)
	end
end

function mNameplates:OnEnable()
	if UnitAffectingCombat("plater") then InCombat = true end	
	
	
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function mNameplates:OnDisable()
	for moduleName, module in self:IterateModules() do
		LibStub("AceAddon-3.0"):DisableAddon(module)
	end
	
	LNP.UnregisterAllCallbacks(self)
end

function mNameplates:InCombat()
	return InCombat
end

function mNameplates:IteratePlates()
	return pairs(Plates)
end

function mNameplates:ShowClickArea(plate)
	if DEBUG then
		expect(plate, "typeof", "table")
		expect(plate.frame, "typeof", "frame")
	end
	if not plate.background then
		local background = self.Objects:GetTexture(plate.frame)
		background:SetAllPoints()
		background:SetTexture(0, 0, 0, 0.5)
		plate.background = background
	end
end

function mNameplates:HideClickArea(plate)
	if DEBUG then
		expect(plate, "typeof", "table")
		expect(plate.frame, "typeof", "frame")
	end
	if plate.background then
		plate.background = plate.background:Delete()
	end
end

function mNameplates:ToggleClickArea()
	if clickAreaVisible then
		for _, plate in self:IteratePlates() do
			self:HideClickArea(plate)
		end
		clickAreaVisible = false
	else
		for _, plate in self:IteratePlates() do
			self:ShowClickArea(plate)
		end
		clickAreaVisible = true
	end
end

function mNameplates:ApplySettings()
	for _, plate in self:IteratePlates() do
		plate:ApplySettings()
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~

function mNameplates:ProfileChanged(event)
	db = self.db.profile
end

local function CastBarShow(self)
	self:Hide()
end

function mNameplates:LibNameplate_NewNameplate(event, plate)
	--Debug:Print("NewPlate", plate)
	
	LNP:GetLevelRegion(plate):Hide()
	local newPlate = Plates[plate] or InactivePlates[plate]
	if not newPlate then
		
		newPlate = setmetatable({
			plate = plate,
			frame = CreateFrame("Frame"),
			modules = {},
		}, PLateProtMT)		
		newPlate.frame:SetAllPoints(plate)

		local healthBar, castBar = plate:GetChildren()		
		local threatGlow, healthBorder, highlight, name, level, bossIcon, raidIcon, eliteIcon = plate:GetRegions()		
		local _, castbarBorder, castNoInterupt, castIcon = castBar:GetRegions()		
		
		if not self.BlizzDefaults then
			local defaults = {
				Height = plate:GetHeight(),
				Width = plate:GetWidth(),
				IconSize = castIcon:GetHeight(),
				ThreatGlowTexture = threatGlow:GetTexture(),
				Highlight = highlight:GetTexture(),
				BossIcon = bossIcon:GetTexture(),
				ElitIcon = eliteIcon:GetTexture(),
				CastBarBorder = castbarBorder:GetTexture(),
				CastNoInterupt = castNoInterupt:GetTexture(),
				RaidIcon = raidIcon:GetTexture();
			}
			self.BlizzDefaults = defaults
		end
		
		healthBar:Hide()
		castBar:Hide()
		name:Hide()		
		healthBorder:Hide()
		
		threatGlow:SetTexture(nil)
		highlight:SetTexture(nil)
		bossIcon:SetTexture(nil)
		eliteIcon:SetTexture(nil)
		castbarBorder:SetTexture(nil)
		castNoInterupt:SetTexture(nil)		
		raidIcon:SetTexture(nil)
		
		castIcon:SetWidth(0.001)
		castIcon:SetHeight(0.001)			
		
		mNameplates:HookScript(castBar, "OnShow", CastBarShow)		
		
		self.Callbacks:Fire("PlateNew", newPlate)
	end	
	
	Plates[plate] = newPlate
	InactivePlates[plate] = nil
	newPlate:ApplySettings()
	
	self.Callbacks:Fire("PlateShow", newPlate)
end

function mNameplates:LibNameplate_RecycleNameplate(event, plate)
	--Debug:Print("RemovePlate", plate)
	local Plate = Plates[plate]
	if Plate then
		if Plate.background then			
			self:HideClickArea(Plate)
		end
		Plates[plate] = nil
		InactivePlates[plate] = Plate
		self.Callbacks:Fire("PlateHide", Plate)
	end
end

function mNameplates:LibNameplate_MouseoverNameplate(event, plate)
	--Debug:Print("MouseOver", plate)
end

function mNameplates:LibNameplate_FoundGUID(event, plate)
	--Debug:Print("GUID", plate)
	plate = Plates[plate]
	if plate then
		self.Callbacks:Fire("PlateGUID", plate, gUID, unit)
	end
end

function mNameplates:LibNameplate_HealthChange(event, plate, health)
	--Debug:Print("HealthChanged", plate)
	plate = Plates[plate]
	if plate then
		self.Callbacks:Fire("PlateHPChanged", plate, health, LNP:GetHealthMax(plate.plate))
	end
end

function mNameplates:LibNameplate_ThreatChange(event, plate, threat)
	--Debug:Print("ThreatChange", plate)
	plate = Plates[plate]
	if plate then
		self.Callbacks:Fire("PlateThreatChange", plate, threat)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:UPDATE_MOUSEOVER_UNIT(...)		
end

function mNameplates:PLAYER_REGEN_DISABLED()
	InCombat = true
	
	--[[if db.ShowHideHostileInCombat then
		SetCVar("nameplateShowEnemies", 1)				
	end
	if db.ShowHideFriendlyInCombat then
		SetCVar("nameplateShowFriends", 1)
	end]]
end

function mNameplates:PLAYER_REGEN_ENABLED()
	InCombat = false
	
	--[[if db.ShowHideHostileInCombat then
		SetCVar("nameplateShowEnemies", 0)				
	end
	if db.ShowHideFriendlyInCombat then
		SetCVar("nameplateShowFriends", 0)
	end]]
end

----------------------------------

function PlateProt:ApplySettings()
	mNameplates:CombatWrapperApply(self.plate)
	mNameplates:CombatWrapperSetStateScript(self.plate)
	
	if clickAreaVisible then
		mNameplates:ShowClickArea(self)
	end
end

function PlateProt:GetData(name)
	return self.modules[name]
end

function PlateProt:SetData(name, data)
	self.modules[name] = data
end

function mNameplates:SetupLDB()
	local LDB = LibStub("LibDataBroker-1.1", true)
	if not LDB then return end
	
	local l = LDB:NewDataObject("mNameplates")
	l.type = "launcher"
	l.icon = [[Interface\Icons\Achievement_BG_AB_defendflags]]
	l.OnClick = function(self, button)
		mNameplates:ToggleConfig()
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine("mNameplates")
		tt:AddLine("Click to toggle configuration")
	end
	
	self.ldbojb = l
end