local name, mNameplates = ...

local DEBUG = mNameplates.DEBUG
local Debug = mNameplates.Debug
local expect = Debug.expect

local plugin = mNameplates:NewModule("HealthBars")
if not plugin then return end

local LNP = LibStub("LibNameplate-1.0")
local LSM = LibStub("LibSharedMedia-3.0")

plugin.DisplayName = "Health Bars"
plugin.DisplayIcon = mNameplates:InterfaceIcon("INV_Ragnaros_Heart")

local db
local Objects = mNameplates.Objects

local defaults = {
	profile = {
		Enabled = true,
		
		ShowFriendly = false,
		Height = 12,		
		StatusBarTexture = LSM:GetDefault("statusbar"),
		
		Border = {
			["**"] = {
				Color = {0, 0, 0, 0.5},
				Texture = "None",
				Inset = 4,
				Width = 5,
			},
			AGGRO = {
				Color = {1, 0 , 0, 1},
			}
		},
		
		Colors = {
			["**"] = {
				["**"] = {
					FRIENDLY = {73/255, 255/255, 81/255, 1},
					NEUTRAL = {255/255, 252/255, 92/255, 1},
					HOSTILE = {255/255, 36/255, 0, 1},
				}
			},
			PLAYER = {
				UseHostileClassColor = true,
				NORMAL = {
					FRIENDLY = {103/255, 134/255, 255/255, 1},
				},
			}
		}
	},
}

function plugin:OnInitialize()	
	self.db = mNameplates.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")	
	
	self:SetEnabledState(db.Enabled)
end

function plugin:OnEnable()
	mNameplates.RegisterCallback(self, "PlateNew")
	mNameplates.RegisterCallback(self, "PlateShow")
	mNameplates.RegisterCallback(self, "PlateHide")
	mNameplates.RegisterCallback(self, "PlateHPChanged")
	mNameplates.RegisterCallback(self, "ModuleStateChanged", "ApplyAllPlates")
	mNameplates.RegisterCallback(self, "PlateThreatChange")
	
	mNameplates.Callbacks:Fire("ModuleStateChanged", self)
end

function plugin:OnDisable()
	mNameplates.UnregisterAllCallbacks(self)
	
	mNameplates.Callbacks:Fire("ModuleStateChanged", self)
end

function plugin:CheckData(data)
	return data.frame ~= nil
end

-- ~~~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~

function plugin:ProfileChanged(event)
	db = self.db.profile
	if db.Enabled then
		if not self:IsEnabled() then self:Enable() end
	elseif not db.Enabled and self:IsEnabled() then 
		self:Disable() 
	end	
end

function plugin:PlateNew(event, plate)
end

function plugin:PlateShow(event, plate)
	local reaction = LNP:GetReaction(plate.plate)
	if reaction == "FRIENDLY" and not db.ShowFriendly then return end
	data = plate:GetData(self:GetName()) or {}
	
	local frame = Objects:GetFrame(plate.frame)
	local bar = Objects:GetStatusBar(frame, "MEDIUM", 2)
	local barBackground = Objects:GetStatusBar(frame, "MEDIUM", 1)
	local border = Objects:GetBorder(bar)
	
	local maxHealth = LNP:GetHealthMax(plate.plate)
	bar:SetMinMaxValues(0, maxHealth)
	bar:SetValue(LNP:GetHealth(plate.plate))
	
	barBackground:SetAllPoints(bar)
	
	data.frame = frame
	data.bar = bar
	data.barBackground = barBackground
	data.border = border
	
	
	plate:SetData(self:GetName(), data)
	
	self:ApplyOnPlate(plate)
end

function plugin:PlateHPChanged(event, plate, health, maxHealth)
	data = plate:GetData(self:GetName())
	if not data or not self:CheckData(data) then return end
	
	data.bar:SetMinMaxValues(0, maxHealth)
	data.bar:SetValue(health)
end

function plugin:PlateHide(event, plate)
	data = plate:GetData(self:GetName())
	if not data or not data.frame then return end
	
	data.frame = data.frame:Delete()
	data.bar = data.bar:Delete()
	data.barBackground = data.barBackground:Delete()
	data.border = data.border:Delete()
end

function plugin:PlateThreatChange(event, plate, threat)
	self:ApplyBorderSettings(plate)
end

-- ~~~~~~~~~~~~~~~~~~~~~~
-- Settings ~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~
function plugin:ApplyBorderSettings(plate)
	data = plate:GetData(self:GetName())
	if not data or not self:CheckData(data) then return end
	
	local threat = LNP:GetThreatSituation(plate.plate)
	local borderDB
	if threat == "HIGH" then
		borderDB = db.Border.AGGRO
	else
		borderDB = db.Border.NORMAL
	end
	data.bar:SetPoint("TOPLEFT", borderDB.Inset, -borderDB.Inset)
	data.bar:SetPoint("BOTTOMRIGHT", -borderDB.Inset, borderDB.Inset)	
	data.border:SetTexture(mNameplates.Media:Get("border", borderDB.Texture))
	data.border:SetWidth(borderDB.Width)
	data.border:SetColor(unpack(borderDB.Color))
	if mNameplates.Media:IsBorderSquare(borderDB.Texture) then
		data.border:SetNormalCoords()
	end
	data.border:SetAnchor(data.frame)
end

function plugin:ApplyOnPlate(plate)
	data = plate:GetData(self:GetName())
	if not data or not self:CheckData(data) then return end
	
	data.frame:SetPoint("BOTTOMLEFT")
	data.frame:SetPoint("BOTTOMRIGHT")
	data.frame:SetHeight(db.Height)
		
	data.bar:SetStatusBarTexture(mNameplates.Media:Get("statusbar", db.StatusBarTexture), db.StatusBarTextureTile)
	data.barBackground:SetStatusBarTexture(mNameplates.Media:Get("statusbar", db.StatusBarTexture), db.StatusBarTextureTile)	
	data.barBackground:SetStatusBarColor(0.1, 0.1, 0.1, 0.7)
	
	self:ApplyBorderSettings(plate)
	
	local unitType = LNP:GetType(plate.plate)
	local reaction = LNP:GetReaction(plate.plate)
	if reaction and unitType then
		if unitType == "NPC" then
			if LNP:IsBoss(plate.plate) then
				data.bar:SetStatusBarColor(unpack(db.Colors.NPC.BOSS[reaction]))
			elseif LNP:IsElite(plate.plate) then
				data.bar:SetStatusBarColor(unpack(db.Colors.NPC.ELITE[reaction]))
			else
				data.bar:SetStatusBarColor(unpack(db.Colors.NPC.NORMAL[reaction]))
			end
		else			
			if (reaction == "HOSTILE" and db.Colors.PLAYER.UseHostileClassColor) then								
				local class = LNP:GetClass(plate.plate)
				if class then
					local c = RAID_CLASS_COLORS[class]
					data.bar:SetStatusBarColor(c.r, c.g, c.b)
				else
					data.bar:SetStatusBarColor(unpack(db.Colors.PLAYER.NORMAL[reaction]))
				end
			else
				data.bar:SetStatusBarColor(unpack(db.Colors.PLAYER.NORMAL[reaction]))
			end
		end
	end
end

function plugin:ApplyAllPlates()
	for i, plate in mNameplates:IteratePlates() do
		self:ApplyOnPlate(plate)
	end
end

local function CopyBorderSettings(from, to)
	to.Color = from.Color
	to.Texture = from.Texture
	to.Width = from.Width
	to.Inset = from.Inset
end

-- ~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~

local options = {
	name = "Health Bars",
	icon = plugin.DisplayIcon,
	groups = {
		{
			name = "Colors",
			children = {
				{
					widget = "ScrollFrame",
					layout = "Relative",
					db = function() return db end,
					callback = function(...)
						plugin:ApplyAllPlates()
					end,
					type = "group",
					layout = "Relative",
					children = {
						{ widget = "CI-InlineGroup", name = "NPCs", type = "group", relwidth = 1, template = "Group2", layout = "Relative",
							children = {
								{ widget = "CI-Label", name = " ", relwidth = 0.25 },
								{ widget = "CI-Label", name = "Friendly", relwidth = 0.25 },
								{ widget = "CI-Label", name = "Neutral", relwidth = 0.25 },
								{ widget = "CI-Label", name = "Hostile", relwidth = 0.25 },
								
								{ widget = "CI-Label", name = "Boss", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.BOSS end, key = "FRIENDLY", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.BOSS end, key = "NEUTRAL", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.BOSS end, key = "HOSTILE", relwidth = 0.25 },
								
								{ widget = "CI-Label", name = "Elite", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.ELITE end, key = "FRIENDLY", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.ELITE end, key = "NEUTRAL", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.ELITE end, key = "HOSTILE", relwidth = 0.25 },
								
								{ widget = "CI-Label", name = "Normal", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.NORMAL end, key = "FRIENDLY", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.NORMAL end, key = "NEUTRAL", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.NPC.NORMAL end, key = "HOSTILE", relwidth = 0.25 },
							},
						},
						{ widget = "CI-InlineGroup", name = "Players", type = "group", relwidth = 1, template = "Group2", layout = "Relative",
							children = {
								{ widget = "CI-Label", name = " ", relwidth = 0.25 },
								{ widget = "CI-Label", name = "Friendly", relwidth = 0.25 },
								{ widget = "CI-Label", name = "Hostile", relwidth = 0.25 },
								{ widget = "CI-Label", name = " ", relwidth = 0.25 },
								
								{ widget = "CI-Label", name = "Normal", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.PLAYER.NORMAL end, key = "FRIENDLY", relwidth = 0.25 },
								{ widget = "CI-ColorSelect", db = function() return db.Colors.PLAYER.NORMAL end, key = "HOSTILE", relwidth = 0.25 },
							},
						},
					},
				},
			},
		}
	},
	children = {
		{
			widget = "ScrollFrame",
			layout = "Relative",
			db = function() return db end,
			callback = function(...)
				plugin:ApplyAllPlates()
			end,
			type = "group",
			children = {			
				{ widget = "CI-CheckBox", name = "Show on friendly plates", key = "ShowFriendly" },
				{ widget = "CI-NewLine" },
				{ widget = "CI-Slider", name = "Height", key = "Height", min = 0, max = 100, step = 0.1, relwidth = 0.5 },
				{ widget = "CI-Dropdown", name = "StatusBar Texture", key = "StatusBarTexture", relwidth = 0.5, items = function() return mNameplates.Media:GetStatusBarItems() end, },
				
				{ widget = "CI-InlineGroup", name = "Border", type = "group", layout = "RelativeBottom", fullwidth = true, db = function() return db.Border.NORMAL end,
					children = {
						{ widget = "CI-Dropdown", name = "Texture", key = "Texture", relwidth = 0.5, items = function() return mNameplates.Media:GetBorderItems() end },																				
						{ widget = "CI-ColorSelect", name = "Border color", key = "Color", relwidth = 0.5 },
						{ widget = "CI-Slider", name = "Border Width", key = "Width", min = 0, max = 20, step = 0.1, relwidth = 0.5 },												
						{ widget = "CI-Slider", name = "Inset", key = "Inset", min = 0, max = 10, step = 0.1, relwidth = 0.5 },						
					}
				},
				{ widget = "CI-Button", name = "Copy ^", callback = function()
					CopyBorderSettings(db.Border.AGGRO, db.Border.NORMAL)
					mNameplates:ToggleConfig(true)
					plugin:ApplyAllPlates()
				end},
				{ widget = "CI-Button", name = "Copy v", callback = function()
					CopyBorderSettings(db.Border.NORMAL, db.Border.AGGRO)
					mNameplates:ToggleConfig(true)
					plugin:ApplyAllPlates()
				end},
				{ widget = "CI-InlineGroup", name = "Border (aggro)", type = "group", layout = "RelativeBottom", fullwidth = true, db = function() return db.Border.AGGRO end,
					children = {
						{ widget = "CI-Dropdown", name = "Texture", key = "Texture", relwidth = 0.5, items = function() return mNameplates.Media:GetBorderItems() end },														
						{ widget = "CI-ColorSelect", name = "Border color", key = "Color", relwidth = 0.5 },
						{ widget = "CI-Slider", name = "Border Width", key = "Width", min = 0, max = 20, step = 0.1, relwidth = 0.5 },									
						{ widget = "CI-Slider", name = "Inset", key = "Inset", min = 0, max = 10, step = 0.1, relwidth = 0.5, disabled = true },
					},
				},				
			}
		}
	}
}

mNameplates:AddDisplayModule(plugin:GetName(), options)

--[[
do
	local function Callback()
		plugin:ApplyAllPlates()
	end
	
	local function Enable()
	end
	
	local function Disable()
	end

	local options
	function plugin:GetOptions()
		if options then return options end
		options = {
			{
				data = {{text = "Modules", value = "Modules"}, {text = self.DisplayName, value = "HealthBars", order = 1}},
				icon = self.DisplayIcon,
				func = function()
					local g = AceGUI:Create("SimpleGroup")
					g:SetLayout("Flow")
					g:SetFullWidth(true)
					
					g:AddChild(UIF:CheckBox("Enabled", db, "Enabled", function() 
						if db.Enabled and not plugin:IsEnabled() then plugin:Enable()
						elseif not db.Enabled and plugin:IsEnabled() then plugin:Disable() end
					end))
					
					local group = UIF:InlineGroup("Position and Size")
					g:AddChild(group)
					group:AddChild(UIF:Slider("X", db, "x", -300, 300, 0.1, Callback, 0.5))
					group:AddChild(UIF:Slider("Y", db, "y", -300, 300, 0.1, Callback, 0.5))
					group:AddChild(UIF:Slider("Width", db, "Width", -300, 300, 0.1, Callback, 0.5))
					group:AddChild(UIF:Slider("Height", db, "Height", -300, 300, 0.1, Callback, 0.5))
					
					local group = UIF:InlineGroup("Colors")
					g:AddChild(group)
					local group2 = UIF:InlineGroup2("NPCs")
					group:AddChild(group2)
					group2:AddChild(UIF:Text1(" ", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Friedly", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Neutral", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Hostile", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Boss", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.BOSS, "FRIENDLY", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.BOSS, "NEUTRAL", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.BOSS, "HOSTILE", Callback, true, 0.25))
					group2:AddChild(UIF:Text1("Elite", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.ELITE, "FRIENDLY", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.ELITE, "NEUTRAL", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.ELITE, "HOSTILE", Callback, true, 0.25))
					group2:AddChild(UIF:Text1("Normal", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.NORMAL, "FRIENDLY", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.NORMAL, "NEUTRAL", Callback, true, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.NPC.NORMAL, "HOSTILE", Callback, true, 0.25))
					
					local group2 = UIF:InlineGroup2("Players")
					group:AddChild(group2)
					group2:AddChild(UIF:CheckBox("Color hostile by class", db.Colors.PLAYER, "UseHostileClassColor", Callback, "Only works in PvP-zones and with the 'Class Colors in Nameplates' option turned on in the Blizzard Interface options"))
					group2:AddChild(UIF:Text1(" ", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Friedly", nil, nil, nil, nil, nil, 0.25))					
					group2:AddChild(UIF:Text1("Hostile", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1(" ", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:Text1("Normal", nil, nil, nil, nil, nil, 0.25))
					group2:AddChild(UIF:ColorSelect("", db.Colors.PLAYER.NORMAL, "FRIENDLY", Callback, true, 0.25))					
					group2:AddChild(UIF:ColorSelect("", db.Colors.PLAYER.NORMAL, "HOSTILE", Callback, true, 0.25))
					group2:AddChild(UIF:Text1(" ", nil, nil, nil, nil, nil, 0.25))
					
					g:AddChild(UIF:StatusbarSelect("Texture", db, "Texture", Callback, 0.5))
					
					return g
				end
			},
		}
		return options
	end
end
]]
