local name, mNameplates = ...

local DEBUG = mNameplates.DEBUG
local Debug = mNameplates.Debug
local expect = Debug.expect

local LSM = LibStub("LibSharedMedia-3.0")
Debug:AddGlobal("LSM", LSM)

local Media = {}
mNameplates.Media = Media
Debug:AddGlobal("Media", Media)

Media.types = {}
Media.types.border = {
	["Beuty Case"] 	= [[Interface\AddOns\mNameplates\Media\beautycase]],
	["Beuty Case (Bright)"] 	= [[Interface\AddOns\mNameplates\Media\beautycase-bright]],
}

function Media:GetBorderItems()
	local borders = {}
	for name, texture in pairs(self.types.border) do
		tinsert(borders, {
			widget = "CI-Dropdown-Item-Border", 
			texture = texture,
			value = name, 
			text = name
		})
	end
	
	for i, v in pairs(LSM.MediaTable.border) do
		tinsert(borders, {
			widget = "CI-Dropdown-Item-Border", 
			texture = v,
			value = i, 
			text = i,
			LSM = true,
		})
	end	
	return borders
end

--- Get a prepared list with statusbar textures to use with dropdowns.
function Media:GetStatusBarItems()
	local statusbars = {}
	for i, v in pairs(LSM.MediaTable.statusbar) do
		tinsert(statusbars, {
			widget = "CI-Dropdown-Item-StatusBar",
			texture = v,
			value = i,
			text = i,
		})
	end
	return statusbars
end

function Media:GetFontItems()
	local fonts = {}
	for i, v in pairs(LSM.MediaTable.font) do
		tinsert(fonts, {
			widget = "CI-Dropdown-Item-Font",
			font = v,
			value = i,
			text = i,
		})
	end
	return fonts
end

function Media:GetOutlineItems()
	local outline = {
		{ widget = "CI-Dropdown-Item", name = "Outline", value = "OUTLINE" },
		{ widget = "CI-Dropdown-Item", name = "Thick Outline", value = "THICKOUTLINE" },
		{ widget = "CI-Dropdown-Item", name = "None", value = "NONE" },
		{ widget = "CI-Dropdown-Item", name = "Monochrome", value = "MONOCHROME" },
		{ widget = "CI-Dropdown-Item", name = "Outline Monochrome", value = "OUTLINE, MONOCHROME" },
		{ widget = "CI-Dropdown-Item", name = "Thick Outline Monochrome", value = "THICKOUTLINE, MONOCHROME" },
	}
	return outline
end

--- Get media of any type
-- @param type Type of media (border, statusbar)
-- @param name Name of the media
function Media:Get(type, name)
	if DEBUG then
		expect(type, "typeof", "string")
		expect(type, "inset", "border;statusbar;font")
		expect(name, "typeof", "string")		
	end
	
	if self.types[type] and self.types[type][name] then
		return self.types[type][name]
	else
		return LSM:Fetch(type, name)
	end
end

function Media:IsBorderSquare(name)
	return self.types["border"][name] ~= nil
end