local name, mNameplates = ...

local DEBUG = mNameplates.DEBUG
local Debug = mNameplates.Debug
local expect = Debug.expect

do
	local taskFuncs = {}
	local taskFrame = CreateFrame("Frame")
	taskFrame:SetScript("OnUpdate", function(self, elapsed)
		for module, functions in pairs(taskFuncs) do
			for func, data in pairs(functions) do
				if data.updateRate then
					data.elapsed = data.elapsed + elapsed
					if data.elapsed > data.updateRate then
						func(module, data.elapsed)
						data.elapsed = 0
					end
				else
					func(module, elapsed)
				end
			end
		end
	end)	
	
	function mNameplates:RegisterTask(module, func, updateRate)
		if DEBUG then
			assert(module and func, "Usage: RegisterTask(module, func)")
		end
		taskFuncs[module] = taskFuncs[module] or {}
		taskFuncs[module][func] = {updateRate = updateRate, elapsed = 0}
	end
	
	function mNameplates:UnregisterTask(module, func)
		if DEBUG then
			assert(module and func, "Usage: UnregisterTask(module, func")
		end
		if taskFuncs[module] then
			taskFuncs[module][func] = nil
		end
	end
	
	function mNameplates:UnregisterAllTasks(module)
		if DEBUG then
			assert(module, "Usage: UnregisterAllTasks(module)")
		end
		taskFuncs[module] = nil
	end
end

function mNameplates:CondensedNumber(value)
	assert(value and type(value) == "number", "Usage CondensedNumber(number)")
	
	local result

	if value > 10000000 then
	   result = ("%.fm"):format(value / 1000000)
	elseif value > 1000000 then
	   result = ("%.2fm"):format(value / 1000000)
	elseif value > 100000 then
	   result = ("%.0fk"):format(value / 1000)
	elseif value > 1000 then
	   result = ("%.1fk"):format(value / 1000)
	end
	
	return result or value
end

function mNameplates:InterfaceIcon(icon)
	if DEBUG then
		expect(icon, "typeof", "string")
	end
	return [[Interface\ICONS\]]..icon
end